import argparse
import validators
import yaml

from litreview.src.shell_utils import get_config_dir


class ArgParseException(Exception):
    def __init__(self, string, type):
        super(ArgParseException, self).__init__(f"Argument {string} could not be cast to {type}")

def str2bool(string):
    string = string.lower()
    if string in ["true", "1", "yes"]:
        return True

    if string in ["false", "0", "no"]:
        return False

    raise ArgParseException(string, "boolean")

def url(string):
    if not validators.url(string):
        raise ArgParseException(string, "url")
    return string

def get_args(configfile="config.yaml"):
    parser = argparse.ArgumentParser()

    try:
        with open(f"{get_config_dir()}/{configfile}", "r") as f:
            conf = yaml.safe_load(f)
        if conf is None:
            conf = {}
    except:
        conf = {}

    def add_argument(argname, type, default=-1):
        conf_argname = argname[2:] if argname.startswith("--") else argname

        if default != -1:
            new_default = type(conf[conf_argname]) if conf_argname in conf else default
            parser.add_argument(argname, type=type, default=new_default)
        else:
            parser.add_argument(argname, type=type)

    add_argument("--verbose", type=str2bool, default=True)
    add_argument("--configfile", type=str, default="config.yaml")
    add_argument("--layout", type=str, default="defaultlayout.md")
    add_argument("--pdfdirname", type=str, default="pdfs")
    add_argument("--postdirname", type=str, default="_posts")
    add_argument("--archive", type=str, default="arxiv")
    add_argument("--overwriteconf", type=str2bool, default=False)

    parser.add_argument("paperurl", type=url)
    parser.add_argument("categories", type=str, nargs="+")

    parsed = parser.parse_args()

    if configfile != "config.yaml":
        return parsed
    if parsed.configfile != "config.yaml":
        return get_args(parsed.configfile)

    return parsed



import datetime

from litreview.src.shell_utils import get_config_dir
import subprocess
import time
import os

def get_post(title, authors, abstract, pdflink, bibtex, date, categories, args, data, format_path=None):
    assert format_path is not None

    with open(f"{get_config_dir()}/{format_path}", "r") as f:
        format = f.read()

    return eval(format).strip()


import subprocess
import pathlib




def run(command, shell=False, capture_output=True):
    r = subprocess.run(command, shell=shell, capture_output=capture_output, check=True, universal_newlines=True)
    return r.stdout

def get_config_dir():
    return run("echo $HOME", shell=True).replace("\n", "") + "/.config/litreview"


def download_pdf(pdflink, filename):
    filepath = f"/tmp/{filename}"
    download = run(
        ["wget", "--user-agent", "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/51.0.2704.103 Safari/537.36", "-O", filepath, pdflink],
        capture_output=False
    )
    return filepath


def copy_file(filepath, newpath):
    return run(["cp", filepath, newpath])



import contextlib
import os
import subprocess


@contextlib.contextmanager
def makedirs(name):
    try:
        os.makedirs(name)
    except:
        pass
    yield None

@contextlib.contextmanager
def directory(name):
    ret = os.getcwd()
    os.chdir(name)
    yield None
    os.chdir(ret)



import os
import subprocess

from setuptools import setup, find_packages
import pathlib

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()


# This call to setup() does all the work
setup(
    name="lit-review",
    version="0.0.7",
    description="Automate creation of reports and working directories for literature reviews",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/Velythyl/litreview",
    author="Charlie Gauthier",
    author_email="charlie.gauthier@umontreal.ca",
    license="MIT",
    packages=find_packages(exclude=("pdfs")),
    package_data={"litreview": ["*.yaml", "*.md"]},
    include_package_data=True,
    install_requires=["pyyaml", "beautifulsoup4", "validators", "six", "soupsieve", "twine"],
    entry_points={
        "console_scripts": [
            "litreview=litreview.main:main",
        ]
    },
)


from litreview.src.argparser import get_args
from litreview.src.extractor import Arxiv
from litreview.src.get_post import get_post
from litreview.src.shell_utils import download_pdf, copy_file, run, get_config_dir
from litreview.src.utils import makedirs


def doit(args):

    data = Arxiv(args.paperurl)

    print(args.postdirname)

    print(data.pdflink())
    print(data.bibtex())
    print(data.abstract())
    print(data.authors())
    print(data.title())
    print(data.date())
    print(args.categories)

    filepath = download_pdf(data.pdflink(), data.filename())

    pdfpath = f"./{args.pdfdirname}/{data.filename()}"
    with makedirs(pdfpath):
        copy_file(filepath, f"{pdfpath}/{data.filename()}.pdf")
        copy_file(filepath, f"{pdfpath}/{data.filename()}_annotated.pdf")

    # with makedirs(args.postdirname):

    the_post = get_post(data.title(), data.authors(), data.abstract(), data.pdflink(), data.bibtex(), data.date(),
                     args.categories, args, data, args.layout)

    import datetime
    with makedirs(f"./{args.postdirname}"):
        with open(f"./{args.postdirname}/{datetime.date.today().strftime('%Y-%m-%d')}-{data.filename()}.md", "w") as f:
            f.write(the_post)

def mkconf(args):
    configdir = get_config_dir()
    from os import path

    if args.overwriteconf:
        run(f"rm -rf {configdir}", shell=True)

    if path.exists(f"{configdir}/config.yaml"):
        return

    import pathlib
    current_path = pathlib.Path(__file__).parent.absolute()

    current_path = str(current_path)

    with makedirs(f"{configdir}"):
        run(f"cp {current_path}/config.yaml {configdir}/", shell=True)
        run(f"cp {current_path}/defaultlayout.md {configdir}/", shell=True)

def main():
    args = get_args()
    mkconf(args)
    doit(args)


if __name__ == "__main__":
    main()


import re

from bs4 import BeautifulSoup
import urllib.request
from urllib.parse import urlparse

# An extractor must return a tuple of this form:
# (<url to download the pdf>, <bibtex>, <title>, <authors>, <abstract>)

def rebuild_url(scheme, netloc, path):
    return scheme+"://"+netloc+path

def eprint2bibtex(eprint_number):
    def soup_from_url(url):
        main_page = urllib.request.urlopen(url).read()
        _url = url
        main_soup = BeautifulSoup(main_page, "html.parser")
        main_soup.prettify()
        return main_soup

    def dblp():
        search = soup_from_url(
            f"https://dblp.uni-trier.de/search?q={eprint_number}"
        ).select(".publ")

        if len(search) > 1:
            print("More than one dblp search result, assuming first is the correct one")

        search = search[0]
        a = search.contents[0].contents[1].contents[0].contents[0]
        bibtex_link = a.attrs["href"]

        bibtex = soup_from_url(
            bibtex_link
        ).select("#bibtex-section")

        if len(bibtex) > 1:
            raise NotImplemented("more than one selected")

        bibtex = bibtex[0].getText().strip()
        return bibtex

    def arxiv():
        bibtex = soup_from_url(
            f"https://arxiv2bibtex.org/?q={eprint_number}&format=bibtex"
        ).select(".wikiinfo")[0]

        bibtex = bibtex.getText().strip()
        return bibtex

    try:
        return dblp()
    except:
        pass

    print("Could not use DBLP to extract bibtex, falling back on arxiv")

    try:
        return arxiv()
    except:
        return "TODO (could not extract bibtex)"



class _Extractor():
    def __init__(self, url):
        self.main_page = urllib.request.urlopen(url).read()
        self._url = url
        self.main_soup = BeautifulSoup(self.main_page, "html.parser")
        self.main_soup.prettify()

    def url(self):
        return self._url

    def pdflink(self):
        raise NotImplemented()

    def bibtex(self):
        raise NotImplemented()

    def abstract(self):
        raise NotImplemented()

    def authors(self):
        raise NotImplemented()

    def title(self):
        raise NotImplemented()

    def date(self):
        raise NotImplemented()

    def filename(self):
        return re.sub(r"\W+", "", self.title())

class Arxiv(_Extractor):
    def pdflink(self):
        parsedurl = urlparse(self.url())

        newpath = parsedurl.path.replace("/abs", "/pdf")+".pdf"
        return rebuild_url(parsedurl.scheme, parsedurl.netloc, newpath)

    def bibtex(self):
        return eprint2bibtex(urlparse(self.url()).path.split("/")[-1])

    def abstract(self):
        selected = self.main_soup.select(".abstract.mathjax")

        if len(selected) > 1:
            raise NotImplemented("more than one selected")

        abstract = str(selected[0].contents[2]).strip()
        return abstract.strip()

    def authors(self):
        selected = self.main_soup.select(".authors")

        if len(selected) > 1:
            raise NotImplemented("more than one selected")

        try:
            selected[0].find("span").replace_with("")
        except:
            pass
        authors = str(selected[0].get_text()).strip()
        return authors

    def date(self):
        selected = self.main_soup.select(".dateline")
        if len(selected) > 1:
            raise NotImplemented("more tha one selected")

        date_text = None
        for i in reversed(range(len(selected[0].contents))):
            date_text = str(selected[0].contents[i])
            matches = re.findall(r"\d\d? [A-Z][a-z][a-z] \d\d\d\d", date_text)
            if len(matches) == 1:
                break

        if date_text is None:
            raise NotImplemented("oh no no date")

        return matches[0].strip()

    def title(self):
        selected = self.main_soup.select(".title.mathjax")

        if len(selected) > 1:
            raise NotImplemented("more than one selected")

        title = str(selected[0].contents[1]).strip()
        return title.strip()

    def filename(self):
        parsedurl = urlparse(self.url())
        return parsedurl.path.replace("/abs/", "").strip()